/*#############################################
#
# Title:        mod_but_authorization.c
# Author:       e1@but.ch
# Date:         February 10, 2006
# Version:      2.0
#
#############################################*/
#include "mod_but.h"

int do_authorization(int shm_offset_number, request_rec *r){

       pcre *re = NULL;  					// the regular expression
       const char *error;				// error text for the failed regex compilation
       int error_offset;				// offset of the regex compilation error, if any
       int rc = 0;					// return code of pcre_exec
       int re_vector[3072];

	mod_but_dir_t *dconfig = ap_get_module_config(r->per_dir_config, &but_module);
	mod_but_server_t *config = ap_get_module_config(r->server->module_config, &but_module);
	if (dconfig == NULL) {
		ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: Illegal Directory Config");
	}

	if(dconfig->logon_required){
		/*
			If we are here, LOGON required is somehow configured (yes, no or some other value)
		*/
		if (!apr_strnatcmp(dconfig->logon_required, "no")){
			ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: MOD_BUT_LOGON_REQUIRED is turned OFF for this Location");
			return 7702;
		}

		if (!apr_strnatcmp(dconfig->logon_required, "yes")){
			static apr_rmm_t *cs_rmm;
			static apr_rmm_off_t *off;
			mod_but_cookie *c;
			ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: MOD_BUT_LOGON_REQUIRED is turned ON for this Location");
			ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: We will check for proper authentication");
			cs_rmm = NULL;
			off = NULL;

			cs_rmm = find_cs_rmm();
			off = find_cs_rmm_off();
			c = apr_rmm_addr_get(cs_rmm, off[shm_offset_number]);

			if (c->logon_state == 0){
				ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: Client not logged in yet (c->logon_state == 0)");
				return 7700;
			}

			if (c->logon_state == 1){
				ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: Client is logged in successfully (c->logon_state == 1)");
				ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: MOD_BUT_LOGON_REQUIRED is configured: Client is logged in successfully (c->logon_state == 1)");
				if(config->service_list_enabled_on){
					ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization: service list check is on");
					ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization: service list access regexp: %s", c->service_list);
					

					if(c->service_list != NULL){
						re = pcre_compile(c->service_list, 0, &error, &error_offset, NULL);
					}else{
						ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: c->service_list PCRE FREE URL STRING IS NULL");
					}
					
					
					if (re == NULL) {
						ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: service_list return code of pcre_compile is NULL");
					}
					
					rc = pcre_exec(re, NULL, r->uri, strlen(r->uri), 0, 0, re_vector, 3072);

					if (rc < 0) {
						ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: service_list PCRE ERROR NOMATCH");
						return 7703;
					}
					
					if (rc < 0 && rc != PCRE_ERROR_NOMATCH) {
						ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: service_list PCRE ERROR NOMATCH");
						return 7703;
					}
					
					if (rc == 0) {
						ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: service_list PCRE output vector too small (%d)", 3072/3-1);
						ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: service_list Problems with the following URI = %s", r->uri);
						return DECLINED;
					}
					
					if (rc > 0) {
						ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: service list PCRE MATCHED!!!");
					}
				}else{
					ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization: service list check is off");
				}
				return 7701;
			}
		}
	}else{
		ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: MOD_BUT_LOGON_REQUIRED is not defined for this URL");
		ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mod_but_authorization.c: MOD_BUT_LOGON_REQUIRED: authentication not required for this URL");
		return 7702;
	}
	return OK;
}

